/*
 * @(#)EventDispatchThread.java	1.14 97/03/10  Amy Fowler
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * Copyright (C) 1997 Microsoft Corporation. Portions copyright (C) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 */

package java.awt;

import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.event.PaintEvent;
import com.ms.awt.WEventQueue;
import java.awt.peer.ActiveEvent;

import java.awt.peer.ComponentPeer;
import com.ms.security.*;
import com.ms.awt.AWTPermission;

/**
 * EventDispatchThread is a package-private AWT class which takes
 * events off the EventQueue and dispatches them to the appropriate
 * AWT components.
 *
 * @version 1.14 03/10/97
 * @author Tom Ball
 * @author Amy Fowler
 */
class EventDispatchThread extends com.ms.lang.SystemThread {
    private EventQueue theQueue;
    private boolean doDispatch = true;

    EventDispatchThread(String name, EventQueue queue) {
		super(name);
        theQueue = queue;
    }

    public void stopDispatching() {
        doDispatch = false;

		if( theQueue instanceof WEventQueue )
		{
			((WEventQueue)theQueue).wakeup();
		}
    }

    public void run() {
       PolicyEngine.denyPermission(AWTPermission.pid);
       while (doDispatch) {
            try {
                AWTEvent event = theQueue.getNextEvent();

                if (false) {
                    // Not until 1.2...
                    // theQueue.dispatchEvent(event);
                } else {
                    // old code...
                    Object src = event.getSource();
                    if (src instanceof Component) {
                        if (event.id == ComponentEvent.COMPONENT_RESIZED) {
                            Component comp = ((ComponentEvent)event).getComponent();
                            ComponentPeer peer = (ComponentPeer)comp.getPeer();
                            if (peer != null) {
                                Rectangle r = Component.getPeerBounds(peer);
                                if (r != null) {
                                    comp.width = r.width;
                                    comp.height = r.height;
                                } 
                            }
                        }
                        ((Component)src).dispatchEvent(event);
                    } else if (src instanceof MenuComponent) {
                        ((MenuComponent)src).dispatchEvent(event);
                    } else if (event instanceof ActiveEvent) {
			// This could become the sole method of dispatching in time, and 
			// moved to the event queue's dispatchEvent() method.
			((ActiveEvent)event).dispatch();
		    }
		}    
            } catch (ThreadDeath death) {
                return;
            } catch (Throwable e) {
                System.err.println(
                    "Exception occurred during event dispatching:");
                e.printStackTrace();
            }
        }
        doDispatch = true;
    }
}

