/*
 * @(#)SystemColor.java	1.7 97/02/20
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * Copyright (C) 1997 Microsoft Corporation. Portions copyright (C) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 */
package java.awt;

/**
 * A class to encapsulate symbolic colors representing the color
 * of GUI objects on a system.  For systems which support the dynamic
 * update of the system colors (when the user changes the colors)
 * the actual RGB values of these symbolic colors will also change
 * dynamically.  In order to compare the "current" RGB value of a SystemColor
 * object with a non-symbolic Color object, getRGB() should be used
 * rather than equals(). 
 *
 * @version 	1.7, 02/20/97
 * @author 	Carl Quinn
 * @author      Amy Fowler
 */
__attribute NAT_L_DCTS()
public final class SystemColor extends Color implements java.io.Serializable {

    /**
     * The array index for the desktop background color.
     */
    public final static int DESKTOP = 0;

    /**
     * The array index for the active caption background color.
     */
    public final static int ACTIVE_CAPTION = 1;

    /**
     * The array index for the action caption text color.
     */
    public final static int ACTIVE_CAPTION_TEXT = 2;

    /**
     * The array index for the active caption border color.
     */
    public final static int ACTIVE_CAPTION_BORDER = 3;

    /**
     * The array index for the inactive caption background color.
     */
    public final static int INACTIVE_CAPTION = 4;

    /**
     * The array index for the inactive caption text color.
     */
    public final static int INACTIVE_CAPTION_TEXT = 5;

    /**
     * The array index for the inactive caption border color.
     */
    public final static int INACTIVE_CAPTION_BORDER = 6;

    /**
     * The array index for the window background color.
     */
    public final static int WINDOW = 7;

    /**
     * The array index for the window border color.
     */
    public final static int WINDOW_BORDER = 8;

    /**
     * The array index for the window text color.
     */
    public final static int WINDOW_TEXT = 9;

    /**
     * The array index for the menu background color.
     */
    public final static int MENU = 10;

    /**
     * The array index for the menu text color.
     */
    public final static int MENU_TEXT = 11;

    /**
     * The array index for the text background color.
     */
    public final static int TEXT = 12;

    /**
     * The array index for the text text color.
     */
    public final static int TEXT_TEXT = 13;

    /**
     * The array index for the text highlight color.
     */
    public final static int TEXT_HIGHLIGHT = 14;

    /**
     * The array index for the text highlight text color.
     */
    public final static int TEXT_HIGHLIGHT_TEXT = 15;

    /**
     * The array index for the text inactive text color.
     */
    public final static int TEXT_INACTIVE_TEXT = 16;

    /**
     * The array index for the control background color.
     */
    public final static int CONTROL = 17;

    /**
     * The array index for the control text color.
     */
    public final static int CONTROL_TEXT = 18;

    /**
     * The array index for the control highlight color.
     */
    public final static int CONTROL_HIGHLIGHT = 19;

    /**
     * The array index for the control light highlight color.
     */
    public final static int CONTROL_LT_HIGHLIGHT = 20;

    /**
     * The array index for the control shadow color.
     */
    public final static int CONTROL_SHADOW = 21;

    /**
     * The array index for the control dark shadow color.
     */
    public final static int CONTROL_DK_SHADOW = 22;

    /**
     * The array index for the scrollbar background color.
     */
    public final static int SCROLLBAR = 23;

    /**
     * The array index for the info background color.
     */
    public final static int INFO = 24;

    /**
     * The array index for the info text color.
     */
    public final static int INFO_TEXT = 25;

    /**
     * The number of system colors in the array.
     */
    public final static int NUM_COLORS = 26;
    
    private final static int COLOR_SCROLLBAR         	= 0;	//23:SCROLLBAR
    private final static int COLOR_BACKGROUND        	= 1;	// 0:DESKTOP
    private final static int COLOR_ACTIVECAPTION     	= 2;	// 1:ACTIVE_CAPTION
    private final static int COLOR_INACTIVECAPTION   	= 3;	// 4:INACTIVE_CAPTION
    private final static int COLOR_MENU              	= 4;	//10:MENU
    private final static int COLOR_WINDOW            	= 5;	// 7:WINDOW
    private final static int COLOR_WINDOWFRAME       	= 6;	// 8:WINDOW_BORDER
    private final static int COLOR_MENUTEXT          	= 7;	//11:MENUTEXT
    private final static int COLOR_WINDOWTEXT        	= 8;	// 9:WINDOW_TEXT
    private final static int COLOR_CAPTIONTEXT       	= 9;	// 2:ACTIVE_CAPTION_TEXT
    private final static int COLOR_ACTIVEBORDER      	= 10;	// 3:ACTIVE_CAPTION_BORDER
    private final static int COLOR_INACTIVEBORDER    	= 11;	// 6:INACTIVE_CAPTION_BORDER
    private final static int COLOR_HIGHLIGHT         	= 13;	//14:TEXT_HIGHLIGHT
    private final static int COLOR_HIGHLIGHTTEXT     	= 14;	//15:TEXT_HIGHLIGHT_TEXT
    private final static int COLOR_3DFACE           	= 15;	//12:TEXT
    													        //17:CONTROL
    private final static int COLOR_3DSHADOW         	= 16;	//21:CONTROL_SHADOW
    private final static int COLOR_GRAYTEXT          	= 17;	//16:TEXT_INACTIVE_TEXT
    private final static int COLOR_BTNTEXT           	= 18;	//13:TEXT_TEXT
    													        //18:CONTROL_TEXT
    private final static int COLOR_INACTIVECAPTIONTEXT 	= 19;	// 5:INACTIVE_CAPTION_TEXT
    private final static int COLOR_3DHIGHLIGHT         	= 20;	//20:CONTROL_LT_HIGHLIGHT
    private final static int COLOR_3DDKSHADOW        	= 21;	//22:CONTROL_DK_SHADOW
    private final static int COLOR_3DLIGHT           	= 22;	//19:CONTROL_HIGHLIGHT
    private final static int COLOR_INFOTEXT          	= 23;	//25:INFO_TEXT
    private final static int COLOR_INFOBK            	= 24;	//24:INFO

    private static int[] sun2Win32 = {
        COLOR_BACKGROUND, 			/* DESKTOP */
        COLOR_ACTIVECAPTION, 		        /* ACTIVE_CAPTION */
        COLOR_CAPTIONTEXT, 			/* ACTIVE_CAPTION_TEXT */
        COLOR_ACTIVEBORDER, 		        /* ACTIVE_CAPTION_BORDER */
        COLOR_INACTIVECAPTION, 		        /* INACTIVE_CAPTION */
        COLOR_INACTIVECAPTIONTEXT, 	        /* INACTIVE_CAPTION_TEXT */
        COLOR_INACTIVEBORDER, 		        /* INACTIVE_CAPTION_BORDER */
        COLOR_WINDOW, 				/* WINDOW */
        COLOR_WINDOWFRAME, 			/* WINDOW_BORDER */
        COLOR_WINDOWTEXT, 			/* WINDOW_TEXT */
        COLOR_MENU, 				/* MENU */
        COLOR_MENUTEXT, 			/* MENU_TEXT */
        COLOR_3DFACE, 				/* TEXT */
        COLOR_BTNTEXT, 				/* TEXT_TEXT */
        COLOR_HIGHLIGHT, 			/* TEXT_HIGHLIGHT */
        COLOR_HIGHLIGHTTEXT, 		        /* TEXT_HIGHLIGHT_TEXT */
        COLOR_GRAYTEXT, 			/* TEXT_INACTIVE_TEXT */
        COLOR_3DFACE, 				/* CONTROL */
        COLOR_BTNTEXT, 				/* CONTROL_TEXT */
        COLOR_3DLIGHT, 				/* CONTROL_HIGHLIGHT */
        COLOR_3DHIGHLIGHT, 			/* CONTROL_LT_HIGHLIGHT */
        COLOR_3DSHADOW, 			/* CONTROL_SHADOW */
        COLOR_3DDKSHADOW, 			/* CONTROL_DK_SHADOW */
        COLOR_SCROLLBAR, 			/* SCROLLBAR */
        COLOR_INFOBK, 				/* INFO */
        COLOR_INFOTEXT, 			/* INFO_TEXT */
    };

   /**
     * The color of the desktop background.
     */
    public final static SystemColor desktop = new SystemColor((byte)DESKTOP);

    /**
     * The background color for captions in window borders.
     */
    public final static SystemColor activeCaption = new SystemColor((byte)ACTIVE_CAPTION);

    /**
     * The text color for captions in window borders.
     */
    public final static SystemColor activeCaptionText = new SystemColor((byte)ACTIVE_CAPTION_TEXT);

    /**
     * The border color for captions in window borders.
     */
    public final static SystemColor activeCaptionBorder = new SystemColor((byte)ACTIVE_CAPTION_BORDER);

    /**
     * The background color for inactive captions in window borders.
     */
    public final static SystemColor inactiveCaption = new SystemColor((byte)INACTIVE_CAPTION);

    /**
     * The text color for inactive captions in window borders.
     */
    public final static SystemColor inactiveCaptionText = new SystemColor((byte)INACTIVE_CAPTION_TEXT);

    /**
     * The border color for inactive captios in window borders.
     */
    public final static SystemColor inactiveCaptionBorder = new SystemColor((byte)INACTIVE_CAPTION_BORDER);

    /**
     * The background color for windows.
     */
    public final static SystemColor window = new SystemColor((byte)WINDOW);

    /**
     * The border color for windows.
     */
    public final static SystemColor windowBorder = new SystemColor((byte)WINDOW_BORDER);

    /**
     * The text color for windows.
     */
    public final static SystemColor windowText = new SystemColor((byte)WINDOW_TEXT);

    /**
     * The background color for menus.
     */
    public final static SystemColor menu = new SystemColor((byte)MENU);

    /**
     * The text color for menus.
     */
    public final static SystemColor menuText = new SystemColor((byte)MENU_TEXT);

    /**
     * The background color for text components.
     */
    public final static SystemColor text = new SystemColor((byte)TEXT);

    /**
     * The text color for text components.
     */
    public final static SystemColor textText = new SystemColor((byte)TEXT_TEXT);

    /**
     * The background color for highlighted text.
     */
    public final static SystemColor textHighlight = new SystemColor((byte)TEXT_HIGHLIGHT);

    /**
     * The text color for highlighted text.
     */
    public final static SystemColor textHighlightText = new SystemColor((byte)TEXT_HIGHLIGHT_TEXT);

    /**
     * The text color for inactive text.
     */
    public final static SystemColor textInactiveText = new SystemColor((byte)TEXT_INACTIVE_TEXT);

    /**
     * The background color for control objects.
     */
    public final static SystemColor control = new SystemColor((byte)CONTROL);

    /**
     * The text color for control objects.
     */
    public final static SystemColor controlText = new SystemColor((byte)CONTROL_TEXT);

    /**
     * The regular highlight color for control objects.
     */
    public final static SystemColor controlHighlight = new SystemColor((byte)CONTROL_HIGHLIGHT);

    /**
     * The light highlight color for control objects.
     */
    public final static SystemColor controlLtHighlight = new SystemColor((byte)CONTROL_LT_HIGHLIGHT);

    /**
     * The regular shadow color for control objects.
     */
    public final static SystemColor controlShadow = new SystemColor((byte)CONTROL_SHADOW);

    /**
     * The dark shadow color for control objects.
     */
    public final static SystemColor controlDkShadow = new SystemColor((byte)CONTROL_DK_SHADOW);

    /**
     * The background color for scrollbars.
     */
    public final static SystemColor scrollbar = new SystemColor((byte)SCROLLBAR);

    /**
     * The background color for info(help) text.
     */
    public final static SystemColor info = new SystemColor((byte)INFO);

    /**
     * The text color for info(help) text.
     */
    public final static SystemColor infoText = new SystemColor((byte)INFO_TEXT);

    /*
     * JDK 1.1 serialVersionUID 
     */
    private static final long serialVersionUID = 4503142729533789064L;
    
    /**
     * Create a symbolic color that represents an indexed entry into system
     * color cache. Used by above static system colors.
     */
    private SystemColor(byte index) {
        super(0, 0, 0);
        value = index;
    }

    /**
     * Gets the "current" RGB value representing the symbolic color.
     * (Bits 24-31 are 0xff, 16-23 are red, 8-15 are green, 0-7 are blue).
     * @see java.awt.image.ColorModel#getRGBdefault
     * @see java.awt.Color#getBlue(int)
     * @see java.awt.Color#getGreen(int)
     * @see java.awt.Color#getRed(int)
     */
    public int getRGB() {
        int syscolor = GetSysColor(sun2Win32[value]);
        return ((syscolor & 0x00FF0000) >> 16) | (syscolor & 0x0000FF00) |
               ((syscolor & 0x000000FF) << 16) | (0xFF000000);
    }

    /**
     * Returns the String representation of this Color's values.
     */
    public String toString() {
        return getClass().getName() + "[i=" + (value) + "]";
    }

    /** @dll.import("USER32") */
    private static native int GetSysColor(int index);
}
