/*
 * @(#)MethodDescriptor.java	1.16 96/12/06  
 * 
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion bdk_beta
 * Copyright (C) 1997 Microsoft Corporation. Portions copyright (C) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 */

package java.beans;

import java.lang.reflect.*;

/**
 * A MethodDescriptor describes a particular method that a Java Bean
 * supports for external access from other components.
 */

public class MethodDescriptor extends FeatureDescriptor {

    /**
     * @param method    The low-level method information.
     */
    public MethodDescriptor(Method method) {
	this.method = method;
	setName(method.getName());
    }


    /**
     * @param method    The low-level method information.
     * @param parameterDescriptors  Descriptive information for each of the
     *		 		method's parameters.
     */
    public MethodDescriptor(Method method, 
		ParameterDescriptor parameterDescriptors[]) {
	this.method = method;
        ensureTearOffsExist();
        tearoffs[TEAROFF_PARAMETERDESCRIPTORS] = parameterDescriptors;
        setName(method.getName());
    }

    /**
     * @return The low-level description of the method
     */
    public Method getMethod() {
	return method;
    }


    /**
     * @return The locale-independent names of the parameters.  May return
     *		a null array if the parameter names aren't known.
     */
    public ParameterDescriptor[] getParameterDescriptors() {
        ParameterDescriptor parameterDescriptors[] = null;
        if (tearoffs != null) {
            parameterDescriptors = (ParameterDescriptor[])
                tearoffs[TEAROFF_PARAMETERDESCRIPTORS];
        }
        return parameterDescriptors;
    }

    /*
     * Package-private constructor
     * Merge two method descriptors.  Where they conflict, give the
     * second argument (y) priority over the first argument (x).
     * @param x  The first (lower priority) MethodDescriptor
     * @param y  The second (higher priority) MethodDescriptor
     */

    MethodDescriptor(MethodDescriptor x, MethodDescriptor y) {
        super(x,y);
        method = x.method;

        //  Note that if either 'x' or 'y' had parameter descriptors, then our
        //  object's tear off array would have been allocated by the super's
        //  constructor.
        if (tearoffs != null) {
            if (y.tearoffs != null) {
                tearoffs[TEAROFF_PARAMETERDESCRIPTORS] =
                    y.tearoffs[TEAROFF_PARAMETERDESCRIPTORS];
            }
            if (tearoffs[TEAROFF_PARAMETERDESCRIPTORS] == null) {
                if (x.tearoffs != null) {
                    tearoffs[TEAROFF_PARAMETERDESCRIPTORS] =
                        x.tearoffs[TEAROFF_PARAMETERDESCRIPTORS];
                }
            }
        }
    }

    static final int TEAROFF_PARAMETERDESCRIPTORS = TEAROFF_EXTERNAL;

    private Method method;
}
