/*
 * @(#)SecurityException.java	1.6 97/01/20
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * Copyright (C) 1997 Microsoft Corporation. Portions copyright (C) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 */
package java.lang;

import com.ms.security.auditing.SecurityAuditor;
import com.ms.security.auditing.ISecurityAuditor;

import java.security.Principal;
import com.ms.security.PolicyEngine;
import com.ms.security.PermissionDataSet;
import com.ms.security.PermissionID;

/**
 * Thrown by the security manager to indicate a security violation. 
 *
 * @author  unascribed
 * @version 1.6, 01/20/97
 * @see     java.lang.SecurityManager
 * @since   JDK1.0
 */
public class SecurityException extends RuntimeException {

    private final void report ()
    {
        PolicyEngine.assertPermission(PermissionID.SYSTEM);
        ISecurityAuditor auditor = SecurityAuditor.getAuditor();
        PolicyEngine.revertPermission(PermissionID.SYSTEM);
        if (auditor != null)
        {
            Class[] skip = new Class[1];
            skip[0] = getClass();
            Class c = PolicyEngine.getClassOfCaller(skip);

            // If there's no class then we were constructed from native.
            Principal p = null;
            if (c != null)
            {
                PermissionDataSet perms = PolicyEngine.getPermissionsOfClass(c);
                if (perms != null && !perms.isFullyTrusted())
                    return;
                p = PolicyEngine.getPrincipalOfClass(c);
            }
            
            auditor.reportAction(ISecurityAuditor.AUDIT_DENIED, p, null, getLocalizedMessage());
        }
    }

    /**
     * Constructs a <code>SecurityException</code> with no detail  message.
     *
     * @since   JDK1.0
     */
    public SecurityException() {
        super();
        report();
    }

    /**
     * Constructs a <code>SecurityException</code> with the specified 
     * detail message. 
     *
     * @param   s   the detail message.
     * @since   JDK1.0
     */
    public SecurityException(String s) {
        super(s);
        report();
    }


    private SecurityException (boolean reported)
    {
        super();
        if (!reported)
            report();
    }

    private SecurityException (boolean reported, String detail)
    {
        super(detail);
        if (!reported)
            report();
    }
}

